#ifndef UI_CORE_WINDOW_H_
#define UI_CORE_WINDOW_H_

#pragma once

namespace ui 
{

class Box;

// Flags for Control::GetControlFlags()
#define UIFLAG_DEFAULT       0x00000000		// Ĭ״̬
#define UIFLAG_TABSTOP       0x00000001		// ʶؼǷյ TAB лʱý

// Flags for FindControl()
#define UIFIND_ALL           0x00000000
#define UIFIND_VISIBLE       0x00000001
#define UIFIND_ENABLED       0x00000002
#define UIFIND_HITTEST       0x00000004
#define UIFIND_TOP_FIRST     0x00000008
#define UIFIND_ME_FIRST      0x80000000


/////////////////////////////////////////////////////////////////////////////////////
//

typedef struct tagTFontInfo
{
	HFONT hFont;
	std::wstring sFontName;
	int iSize;
	bool bBold;
	bool bUnderline;
	bool bItalic;
	TEXTMETRIC tm;
} TFontInfo;

// MessageFilter interface
class IUIMessageFilter
{
public:
	virtual LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) = 0;
};

class ITranslateAccelerator
{
public:
	virtual LRESULT TranslateAccelerator(MSG *pMsg) = 0;
};
class IControlFromPointFinder
{
public:
	virtual Control* FindControlFromPoint(const CPoint& pt) = 0;
};
/////////////////////////////////////////////////////////////////////////////////////
//
#ifndef ASSERT
#define ASSERT(expr)  _ASSERTE(expr)
#endif

/////////////////////////////////////////////////////////////////////////////////////
//

class UILIB_API Window : public virtual nbase::SupportWeakCallback
{
public:
    Window();
	~Window();

	/**
	 * @brief ȡ Windows 
	 * @return شڹ Windows ھ
	 */
    HWND GetHWND() const;

	/**
	 * @brief עᴰ
	 * @return  true ʾɹʾʧ
	 */
    bool RegisterWindowClass();

	/**
	 * @brief עؼࣨ봰ڵĹ̺ͬ
	 * @return  true ʾɹʾʧ
	 */
    bool RegisterSuperClass();

	/**
	 * @brief ȡ
	 * @return ش
	 */
	virtual std::wstring GetWindowClassName() const;

	/**
	 * @brief ȡؼ
	 * @return ؿؼ
	 */
	virtual std::wstring GetSuperClassName() const;

	/**
	 * @brief ȡʽ÷ʵʵ֣https://docs.microsoft.com/en-us/windows/desktop/winmsg/window-class-styles
	 * @return شʽ
	 */
	virtual UINT GetClassStyle() const;

	/**
	 * @brief ໯ڣĴڹ̺
	 * @param[in] ھ
	 * @return µĴھ
	 */
	HWND Subclass(HWND hWnd);

	/**
	 * @brief ȡ໯ڣָԭĴڹ̺
	 * @return 
	 */
	void Unsubclass();

	/**
	 * @brief 
	 * @param[in] hwndParent ھ
	 * @param[in] pstrName 
	 * @param[in] dwStyle ʽ
	 * @param[in] dwExStyle չʽ
	 * @param[in] isLayeredWindow Ƿв㴰ԣĬΪ true
	 * @param[in] rc ڴС
	 * @return شھ
	 */
	virtual HWND Create(HWND hwndParent, LPCTSTR pstrName, DWORD dwStyle, DWORD dwExStyle, bool isLayeredWindow = true, const UiRect& rc = UiRect(0, 0, 0, 0));

	/**
	 * @brief رմ
	 * @param[in] رϢ
	 * @return 
	 */
	virtual void Close(UINT nRet = IDOK);

	/**
	 * @brief ʾش
	 * @param[in] bShow Ϊ true ʱʾڣΪ false ʱΪشڣĬΪ true
	 * @param[in] bTakeFocus Ƿý㣨ڣĬΪ true
	 * @return 
	 */
	virtual void ShowWindow(bool bShow = true, bool bTakeFocus = true);
	
	/**
	 * @brief ʾģ̬ԻƼ
	 * @param[in] parent_hwnd ھ
	 * @return 
	 */
	void ShowModalFake(HWND parent_hwnd);

	/**
	 * @brief дڣ֧չĻ
	 * @return 
	 */
    void CenterWindow();	

	/**
	 * @brief ôͼ
	 * @param[in] nRes ͼԴ ID
	 * @return 
	 */
    void SetIcon(UINT nRes);

	/**
	 * @brief Ϣ Windows SendMessage һװ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return شڶϢĴ
	 */
    LRESULT SendMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = 0L);

	/**
	 * @brief ͶһϢϢ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return شڶϢĴ
	 */
    LRESULT PostMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = 0L);

	/**
	 * @brief ڹر¼
	 * @param[in] callback ָرպĻص
	 * @return 
	 */
	void AttachWindowClose(const EventCallback& callback);

protected:
	/**
	 * @brief ڴյ WM_NCDESTROY Ϣʱᱻ
	 * @param[in] hWnd ھ
	 * @return 
	 */
	virtual void OnFinalMessage(HWND hWnd);

	/**
	 * @brief ڹ̺
	 * @param[in] hWnd ھ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return Ϣ
	 */
	static LRESULT CALLBACK __WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/**
	 * @brief 
	 * @param[in] 
	 * @return 
	 */
	static LRESULT CALLBACK __ControlProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	/**
	 * @brief ڽյ WM_CREATE Ϣʱᱻ
	 * @param[in] hWnd ھ
	 * @return 
	 */
	void Init(HWND hWnd);

	/**
	 * @brief 󶨴ڵĶ
	 * @param[in] pRoot ָ
	 * @return  true 󶨳ɹΪʧ
	 */
	bool AttachDialog(Box* pRoot);

	/**
	 * @brief ʼؼӿؼʱᱻãڶԿؼ棩
	 * @param[in] pControl ؼָ
	 * @param[in] pParent ؼ
	 * @return true ΪɹΪʧ
	 */
	bool InitControls(Control* pControl, Box* pParent = NULL);

	/**
	 * @brief տؼ
	 * @param[in] pControl ؼָ
	 * @return 
	 */
	void ReapObjects(Control* pControl);

	/// Դز
	/**
	 * @brief ȡԴ·
	 * @return شԴ·
	 */
	std::wstring GetWindowResourcePath();

	/**
	 * @brief ôԴ·
	 * @param[in] strPath Ҫõ·
	 * @return 
	 */
	void SetWindowResourcePath(const std::wstring& strPath);

	/**
	 * @brief ȡĬϢ
	 * @return ĬϢṹ
	 */
	TFontInfo* GetDefaultFontInfo();

	/**
	 * @brief һͨʽ
	 * @param[in] strClassName ͨʽ
	 * @param[in] strControlAttrList ͨʽ XML תʽ
	 * @return 
	 */
	void AddClass(const std::wstring& strClassName, const std::wstring& strControlAttrList);

	/**
	 * @brief ȡͨʽ
	 * @return ͨʽ map 
	 */
	const std::map<std::wstring, std::wstring>* GetClassMap();

	/**
	 * @brief ȡָͨʽ
	 * @param[in] strClassName ͨʽ
	 * @return ָƵͨʽݣXML תʽ
	 */
	std::wstring GetClassAttributes(const std::wstring& strClassName) const;

	/**
	 * @brief ɾһͨʽ
	 * @param[in] strClassName Ҫɾͨʽ
	 * @return  true Ϊɹfalse Ϊʧܻʽ
	 */
	bool RemoveClass(const std::wstring& strClassName);

	/**
	 * @brief ɾͨʽ
	 * @return 
	 */
	void RemoveAllClass();

	/**
	 * @brief һѡ
	 * @param[in] strGroupName 
	 * @param[in] pControl ؼָ
	 * @return  true ʾӳɹfalse Ѿ
	 */
	bool AddOptionGroup(const std::wstring& strGroupName, Control* pControl);

	/**
	 * @brief ȡָѡпؼб
	 * @param[in] strGroupName ָ
	 * @return ظµпؼб
	 */
	std::vector<Control*>* GetOptionGroup(const std::wstring& strGroupName);

	/**
	 * @brief ɾһѡ
	 * @param[in] strGroupName 
	 * @param[in] pControl ؼ
	 * @return 
	 */
	void RemoveOptionGroup(const std::wstring& strGroupName, Control* pControl);

	/**
	 * @brief ɾѡ
	 * @return 
	 */
	void RemoveAllOptionGroups();

	/**
	 * @brief ͼƬ
	 * @return 
	 */
	void ClearImageCache();

	/// 
	/**
	 * @brief ȡλ
	 * @return λõ x  y 
	 */
	POINT GetMousePos() const;

	/**
	 * @brief ȡı߿췶ΧĴС
	 * @return شı߿췶ΧĴС
	 */
	UiRect GetSizeBox();

	/**
	 * @brief ôı߿췶ΧĴС
	 * @param[in] rcSizeBox ҪõĴС
	 * @return 
	 */
	void SetSizeBox(const UiRect& rcSizeBox);

	/**
	 * @brief ȡڱ򣨿϶򣩣Ӧ XML  caption 
	 * @return ر
	 */
	UiRect GetCaptionRect() const;

	/**
	 * @brief ôڱ
	 * @param[in] rcCaption ҪõΧ
	 * @return 
	 */
	void SetCaptionRect(UiRect& rcCaption);

	/**
	 * @brief ȡԲǴСӦ XML  roundcorner 
	 * @return ԲǴС
	 */
	CSize GetRoundCorner() const;

	/**
	 * @brief ôԲǴС
	 * @param[in] cx Բǿ
	 * @param[in] cy ԲǸ
	 * @return 
	 */
	void SetRoundCorner(int cx, int cy);

	/**
	 * @brief ȡϢ
	 * @return شϢ
	 */
	UiRect GetMaximizeInfo() const;

	/**
	 * @brief ôϢ
	 * @param[in] rcMaximize Ҫõֵ
	 * @return 
	 */
	void SetMaximizeInfo(UiRect& rcMaximize);

	/**
	 * @brief ȡ͸ͨ޲ΧĵľŹӦ XML  alphafixcorner 
	 * @return ޲Χ
	 */
	UiRect GetAlphaFixCorner() const;

	/**
	 * @brief ͸ͨ޲ΧĵľŹ
	 * @param[in] rc Ҫõ޲Χ
	 * @return 
	 */
	void SetAlphaFixCorner(UiRect& rc);

	/**
	 * @brief ȡڵĳʼ߶ռĻ߶ȵİٷֱȣӦ XML  heightpercent 
	 * @return ָٷֱ
	 */
	double GetHeightPercent() const;

	/**
	 * @brief ôڵĳʼ߶ռĻ߶ȵİٷֱ
	 * @param[in] heightPercent ٷֱ
	 * @return 
	 */
	void SetHeightPercent(double heightPercent);

	/**
	 * @brief беı ID ôڱı
	 * @param[in] strTextId  ID ID ļд
	 * @return 
	 */
	void SetTextId(const std::wstring& strTextId);

	/// Ӱز
	/**
	 * @brief ôǷ񸽼ӰЧ
	 * @param[in] bShadowAttached Ϊ true ʱӣfalse ʱ
	 * @return 
	 */
	void SetShadowAttached(bool bShadowAttached);

	/**
	 * @brief ȡӰͼƬ
	 * @return ӰͼƬλ
	 */
	std::wstring GetShadowImage() const;

	/**
	 * @brief ôӰͼƬ
	 * @param[in] strImage ͼƬλ
	 * @return 
	 */
	void SetShadowImage(const std::wstring &strImage);

	/**
	 * @brief ȡӰľŹϢ
	 * @return ӰľŹϢ
	 */
	UiRect GetShadowCorner() const;

	/**
	 * @brief ָӰزĵľŹ
	 * @param[in] rect ŹϢ
	 * @return 
	 */
	void SetShadowCorner(const UiRect rect);

	/**
	 * @brief ȡλϢ
	 * @param[in] bContainShadow ǷӰtrue ΪĬΪ false 
	 * @return شλϢ
	 */
	UiRect GetPos(bool bContainShadow = false) const;

	/**
	 * @brief ôλã SetWindowPos һװ
	 * @param[in] rc λ
	 * @param[in] bNeedDpiScale Ϊ false ʾҪ rc  DPI Զ
	 * @param[in] uFlags Ӧ SetWindowPos  uFlags ѡ
	 * @param[in] hWndInsertAfter Ӧ SetWindowPos  hWndInsertAfter ѡ
	 * @param[in] bContainShadow ǷӰΧĬΪ false
	 * @return 
	 */
	void SetPos(const UiRect& rc, bool bNeedDpiScale, UINT uFlags, HWND hWndInsertAfter = NULL, bool bContainShadow = false);

	/**
	 * @brief ȡСΧӦ XML  mininfo 
	 * @param[in] bContainShadow ǷӰΧĬΪ false
	 * @return ָС
	 */
	CSize GetMinInfo(bool bContainShadow = false) const;

	/**
	 * @brief ȡΧӦ XML  maxinfo 
	 * @param[in] bContainShadow ǷӰΧĬΪ false
	 * @return ָС
	 */
	CSize GetMaxInfo(bool bContainShadow = false) const;

	/**
	 * @brief ȡڳʼС
	 * @param[in] bContainShadow ǷӰΧĬΪ false
	 * @return ָС
	 */
	CSize GetInitSize(bool bContainShadow = false) const;

	/**
	 * @brief ôСΧ
	 * @param[in] cx 
	 * @param[in] cy ߶
	 * @param[in] bContainShadow Ϊ false ʾ cx cy Ӱ
	 * @return 
	 */
	// 
	void SetMinInfo(int cx, int cy, bool bContainShadow = false);

	/**
	 * @brief ôΧ
	 * @param[in] cx 
	 * @param[in] cy ߶
	 * @param[in] bContainShadow Ϊ false ʾ cx cy Ӱ
	 * @return 
	 */
	void SetMaxInfo(int cx, int cy, bool bContainShadow = false);

	/**
	 * @brief ôڳʼС
	 * @param[in] cx 
	 * @param[in] cy ߶
	 * @param[in] bContainShadow Ϊ false ʾ cx cy Ӱ
	 * @param[in] bNeedDpiScale Ϊ false ʾ DPI 
	 * @return 
	 */
	void SetInitSize(int cx, int cy, bool bContainShadow = false, bool bNeedDpiScale = true);

	/**
	 * @brief һϢʱϢѾɷ
	 * @param[in] pFilter һ̳ IUIMessageFilter ĶʵҪʵ MessageHandler 
	 * @return ʼշ true
	 */
	bool AddMessageFilter(IUIMessageFilter* pFilter);

	/**
	 * @brief ƳһϢ
	 * @param[in] pFilter һ̳ IUIMessageFilter Ķʵ
	 * @return  true ʾƳɹܸù
	 */
	bool RemoveMessageFilter(IUIMessageFilter* pFilter);

	/**
	 * @brief ҿؼʱһλòҿؼĹ
	 * @param[in] pFinder ̳ IControlFromPointFinder ָ룬ʵ FindControlFromPoint 
	 * @return ʼշ true
	 */
	bool AddControlFromPointFinder(IControlFromPointFinder* pFinder);

	/**
	 * @brief ƳҿؼʱλòҿؼĹ
	 * @param[in] pFinder ̳ IControlFromPointFinder ָ
	 * @return ɹ trueùӿܲ
	 */
	bool RemoveControlFromPointFinder(IControlFromPointFinder* pFinder);

	/**
	 * @brief һ TranslateMessage ֮ǰϢ
	 * @param[in] pTranslateAccelerator ̳ ITranslateAccelerator ָ룬ʵ TranslateAccelerator 
	 * @return ʼշ true
	 */
	bool AddTranslateAccelerator(ITranslateAccelerator *pTranslateAccelerator);

	/**
	 * @brief Ƴһ TranslateMessage ֮ǰϢ
	 * @param[in] pTranslateAccelerator ̳ ITranslateAccelerator ָ
	 * @return ɹ true򷵻 false ܸù
	 */
	bool RemoveTranslateAccelerator(ITranslateAccelerator *pTranslateAccelerator);

	/**
	 * @brief ִ TranslateMessage ׶εĹ
	 * @param[in] pMsg Ϣ
	 * @return  true ɹϢ򷵻 false
	 */
	bool TranslateAccelerator(LPMSG pMsg);

	/**
	 * @brief Ϣɷ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return  true ɷϢɷϢ
	 */
	virtual LRESULT HandleMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/**
	 * @brief Ϣִ壬ִиʹϢ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @param[out] handled ϢǷѾ true ѾɹϢϢݸڹ
	 * @return Ϣ
	 */
	LRESULT DoHandlMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, bool& handled);

	/**
	 * @brief  CallWindowProc API һװ
	 * @param[in] uMsg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return Ϣ
	 */
	LRESULT CallWindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/**
	 * @brief жǷҪ뿪Ϣ
	 * @param[in] pt 굱ǰλ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return  true Ҫ뿪Ϣ false ΪҪ
	 */
	inline bool HandleMouseEnterLeave(const POINT &pt, WPARAM wParam, LPARAM lParam);

	/// 
	/**
	 * @brief ȡǰнĿؼ
	 * @return ؿؼָ
	 */
	Control* GetFocus() const;

	/**
	 * @brief ý㵽ָؼ
	 * @param[in] pControl ؼָ
	 * @return 
	 */
	void SetFocus(Control* pControl);

	/**
	 * @brief ý㵽ָؼػпؼ
	 * @param[in] pControl ؼָ
	 * @return 
	 */
	void SetFocusNeeded(Control* pControl);

	/**
	 * @brief ôʧȥ
	 * @return 
	 */
	void KillFocus();

	/**
	 * @brief õҪ괰ھΪǰƴ
	 * @param[in]
	 * @return 
	 */
	void SetCapture();

	/**
	 * @brief ڲҪʱͷԴ
	 * @return 
	 */
	void ReleaseCapture();

	/**
	 * @brief жָؼǷ񱻵򴥿
	 * @paran[in] pControl ؼָ
	 * @return  true ʾǰؼڱ򴥿״̬
	 */
	bool IsCaptureControl(const Control* pControl);

	/**
	 * @brief жϵǰǷ񲶻
	 * @return  true Ϊ񣬷Ϊ
	 */
	bool IsCaptured() const;

	/**
	 * @brief ȡǰĸؼ
	 * @return ؿؼָ
	 */
	Control* GetNewHover();

	/**
	 * @brief ȡ
	 * @return Ϣ
	 */
	POINT GetLastMousePos() const;

	/**
	 * @brief ǷϢ
	 * @param[in] bHandle Ϊ true ʾfalse Ϊ
	 * @return 
	 */
	void SetHandlePointer(bool bHandle) { m_bHandlePointer = bHandle; };

	/**
	 * @brief ȡʾϢĴھ
	 * @return ʾϢĴھ
	 */
	HWND GetTooltipWindow() const;

	/**
	 * @brief лؼ㵽һһؼ
	 * @param[in] bForward true ΪһؼΪ falseĬΪ true
	 * @return ʼշ true޲ο
	 */
	bool SetNextTabControl(bool bForward = true);

	/// ؼ
	/**
	 * @brief ȡ
	 * @return 
	 */
	Control* GetRoot() const;

	/**
	 * @brief ÿؼǷѾ
	 * @param[in] bArrange true ΪѾУΪ false
	 * @return 
	 */
	void SetArrange(bool bArrange);

	/**
	 * @brief ӳһؼ
	 * @param[in] pControl ؼָ
	 * @return 
	 */
	void AddDelayedCleanup(Control* pControl);

	/**
	 * @brief ָؼ
	 * @param[in] pt ָ
	 * @return ؿؼָ
	 */
	Control* FindControl(POINT pt) const;

	/**
	 * @brief ݿؼƲҿؼ
	 * @param[in] strName ؼ
	 * @return ؿؼָ
	 */
	Control* FindControl(const std::wstring& strName) const;

	/**
	 * @brief ӿؼ
	 * @param[in] pParent ҪĿؼ
	 * @param[in] pt Ҫҵ
	 * @return ؿؼָ
	 */
	Control* FindSubControlByPoint(Control* pParent, POINT pt) const;

	/**
	 * @brief ֲӿؼ
	 * @param[in] pParent ҪĿؼ
	 * @param[in] strName Ҫҵ
	 * @return ؿؼָ
	 */
	Control* FindSubControlByName(Control* pParent, const std::wstring& strName) const;

	/**
	 * @brief ӿؼ
	 * @param[in] pParent ҪĿؼ
	 * @param[in] typeinfo ͵Ϣ
	 * @param[in] iIndex ҵжҪȡڼ
	 * @return ؼָ
	 */
	Control* FindSubControlByClass(Control* pParent, const type_info& typeinfo, int iIndex = 0);

	/**
	 * @brief ӿؼ
	 * @param[in] pParent ҪĿؼ
	 * @param[in] typeinfo ͵Ϣ
	 * @return зĿؼб
	 */
	std::vector<Control*>* FindSubControlsByClass(Control* pParent, const type_info& typeinfo);

	/**
	 * @brief ر˿ؼϵб
	 * @return б
	 */
	std::vector<Control*>* GetSubControlsByClass();

	/**
	 * @brief һϢ
	 * @param[in] eventType תϢ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return ʼշ true
	 */
	bool SendNotify(EventType eventType, WPARAM wParam = 0, LPARAM lParam = 0);

	/**
	 * @brief һϢ
	 * @param[in] pControl Ŀ꣬Ҫ͸ĸؼ
	 * @param[in] msg Ϣ
	 * @param[in] wParam ϢӲ
	 * @param[in] lParam ϢӲ
	 * @return ʼշ true
	 */
	bool SendNotify(Control* pControl, EventType msg, WPARAM wParam = 0, LPARAM lParam = 0);

	/// 
	/**
	 * @brief ȡ DC
	 * @return ػ DC
	 */
	HDC GetPaintDC() const;

	/**
	 * @brief ȡƶ
	 * @return ػƶ
	 */
	IRenderContext* GetRenderContext() const;

	/**
	 * @brief ػϢ
	 * @param[in] rcItem ػ淶Χ
	 * @return 
	 */
	void Invalidate(const UiRect& rcItem);

	/**
	 * @brief ƺ
	 * @return 
	 */
	void Paint();

	/**
	 * @brief ͸
	 * @param[in] nAlpha ͸ֵ
	 * @return 
	 */
	void SetAlpha(int nAlpha);

	/**
	 * @brief жϵǰǷȾ͸ͼ
	 * @return  true ΪǣΪ false
	 */
	bool IsRenderTransparent() const;

	/**
	 * @brief Ⱦ͸ͼ
	 * @param[in] bCanvasTransparent  true ΪȾ͸ͼ㣬Ϊ false
	 * @return һλ״̬
	 */
	bool SetRenderTransparent(bool bCanvasTransparent);

	/**
	 * @brief ʼ
	 * @return 
	 */
	virtual void OnInitLayout();

private:
	static Control* CALLBACK __FindControlFromNameHash(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromCount(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromPoint(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromTab(Control* pThis, LPVOID pData);
	//static Control* CALLBACK __FindControlFromShortcut(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromUpdate(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromName(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlFromClass(Control* pThis, LPVOID pData);
	static Control* CALLBACK __FindControlsFromClass(Control* pThis, LPVOID pData);

private:
	/**
	 * @brief ûƫ
	 * @param[in] renderOffset ƫֵ
	 * @return 
	 */
	void SetRenderOffset(CPoint renderOffset);

	/**
	 * @brief ûƫ x 
	 * @param[in] renderOffsetX ֵ
	 * @return 
	 */
	void SetRenderOffsetX(int renderOffsetX);

	/**
	 * @brief ûƫ y 
	 * @param[in] renderOffsetY ֵ
	 * @return 
	 */
	void SetRenderOffsetY(int renderOffsetY);

protected:
	HWND m_hWnd;
	WNDPROC m_OldWndProc;
	bool m_bSubclassed;

	Box* m_pRoot;
	EventMap OnEvent;

protected:
	CSize m_szMinWindow;
	CSize m_szMaxWindow;
	CSize m_szInitWindowSize;
	UiRect m_rcMaximizeInfo;
	UiRect m_rcSizeBox;
	UiRect m_rcAlphaFix;
	CSize m_szRoundCorner;
	UiRect m_rcCaption;
	double m_heightPercent;

	HDC m_hDcPaint;
	std::unique_ptr<IRenderContext> m_renderContext;
	bool m_bIsLayeredWindow;
	int m_nAlpha;
	CPoint m_renderOffset;
	bool m_bFirstLayout;

	HWND m_hwndTooltip;
	TOOLINFO m_ToolTip;

	Control* m_pFocus;
	Control* m_pNewHover;
	Control* m_pEventHover;
	Control* m_pEventClick;
	Control* m_pEventKey;
	CPoint m_ptLastMousePos;

	Control* m_pEventTouch;
	CPoint m_ptLastTouchPos;
	Control* m_pEventPointer;
	bool m_bHandlePointer;

	UINT m_uTimerID;
	bool m_bIsArranged;
	bool m_bFocusNeeded;
	bool m_bMouseTracking;
	bool m_bMouseCapture;

	std::wstring m_strWindowResourcePath; //ÿڵԴ·,GetSkinFolder()
	TFontInfo m_defaultFontInfo;
	std::map<std::wstring, std::wstring> m_defaultAttrHash;
	std::map<std::wstring, std::vector<Control*>> m_mOptionGroup;

	std::vector<IUIMessageFilter*> m_aPreMessageFilters;
	std::vector<IUIMessageFilter*> m_aMessageFilters;
	std::vector<ITranslateAccelerator*> m_aTranslateAccelerator;
	std::vector<IControlFromPointFinder*> m_aIControlFromPointFinder;

	std::vector<Control*> m_aDelayedCleanup;
	std::vector<Control*> m_aFoundControls;
	std::map<std::wstring, Control*> m_mNameHash;

	nbase::WeakCallbackFlag m_closeFlag;
	
	Shadow m_shadow;

	bool m_bFakeModal = false;
};

} // namespace ui

#endif // UI_CORE_WINDOW_H_
