#pragma once
#include "include/cef_app.h"
#include "base/memory/singleton.h"

namespace nim_comp
{

typedef std::function<void(bool has_error, const std::string& result)> ReportResultFunction;

typedef std::function<void(const std::string& result)> CallJsFunctionCallback;
typedef std::function<void(const std::string& params, ReportResultFunction callback)> CppFunction;

typedef std::map<int/* js_callback_id*/, std::pair<CefRefPtr<CefV8Context>/* context*/, CefRefPtr<CefV8Value>/* callback*/>> RenderCallbackMap;
typedef std::map<int/* cpp_callback_id*/, CallJsFunctionCallback/* callback*/> BrowserCallbackMap;

typedef std::map<std::pair<CefString/* function_name*/, int64_t/* frame_id*/>, CefRefPtr<CefV8Value>/* function*/> RenderRegisteredFunction;
typedef std::map<std::pair<CefString/* function_name*/, int64_t/* browser_id*/>, CppFunction/* function*/> BrowserRegisteredFunction;

class CefJSBridge
{
public:
	CefJSBridge();
	~CefJSBridge();

// in render process
public:
	/**
	 * ִѾעõ C++ 
	 * param[in] function_name	Ҫõĺ
	 * param[in] params			øúݵ json ʽ
	 * param[in] callback		ִɺĽص
	 * return  true ʾִɹһִгɹ忴ص false עĻص ID Ѿ
	 */
	bool CallCppFunction(const CefString& function_name, const CefString& params, CefRefPtr<CefV8Value> callback);

	/**
	 * ͨжĻƳָصҳˢ»ᴥ÷
	 * param[in] frame		ǰп
	 */
	void RemoveCallbackFuncWithFrame(CefRefPtr<CefFrame> frame);

	/**
	 *  ID ִָص
	 * param[in] js_callback_id	ص ID
	 * param[in] has_error		Ƿд󣬶Ӧصһ
	 * param[in] json_string	ûд򷵻ָ json string ʽݣӦصڶ
	 * return  true ʾɹִ˻ص false пܻصڻ߻صִĻѾ
	 */
	bool ExecuteJSCallbackFunc(int js_callback_id, bool has_error, const CefString& json_result);

	/**
	 * עһ־õ JS ṩ C++ 
	 * param[in] function_name	ƣַʽṩ C++ ֱӵãƲظ
	 * param[in] context		ִĻ
	 * param[in] function		
	 * param[in] replace		ѾڸƵĺǷ滻ĬϷ
	 * return replace Ϊ true £ true 滻ɹ false ΪԤΪreplace Ϊ false · true ʾעɹ false ͬѾעˡ
	 */
	bool RegisterJSFunc(const CefString& function_name, CefRefPtr<CefV8Value> function, bool replace = false);

	/**
	 * עһ־õ JS 
	 * param[in] function_name	
	 * param[in] frame			Ҫȡעĸµغ
	 */
	void UnRegisterJSFunc(const CefString& function_name, CefRefPtr<CefFrame> frame);

	/**
	* ִķעһ־õ JS 
	* param[in] frame			ǰ
	*/
	void UnRegisterJSFuncWithFrame(CefRefPtr<CefFrame> frame);

	/**
	 * ִĳ JS 
	 * param[in] function_name	
	 * param[in] json_params	Ҫݵ json ʽĲ
	 * param[in] frame			ִĸµ JS 
	 * param[in] cpp_callback_id	ִɺҪص C++ ص ID
	 * return  true ʾɹִĳ JS  false пҪִеĺڻ߸úѾЧ
	 */
	bool ExecuteJSFunc(const CefString& function_name, const CefString& json_params, CefRefPtr<CefFrame> frame, int cpp_callback_id);

// in browser process
public:
	/**
	 * ִѾעõ JS 
	 * param[in] js_function_name Ҫõ JS 
	 * param[in] params			 JS ݵ json ʽ
	 * param[in] frame			ĸµ JS 
	 * param[in] callback		 JS 󷵻ݵĻص
	 * return  ture ʶִ JS ɹ false ͬ callback id Ѿ
	 */
	bool CallJSFunction(const CefString& js_function_name, const CefString& params, CefRefPtr<CefFrame> frame, CallJsFunctionCallback callback);

	/**
	 *  ID ִָص
	 * param[in] cpp_callback_id callback  id
	 * param[in] json_string	ص json ʽ
	 * return  true ִгɹfalse Ϊִʧܣܻص
	 */
	bool ExecuteCppCallbackFunc(int cpp_callback_id, const CefString& json_string);

	/**
	 * עһ־õ C++ ṩ JS ˵
	 * param[in] function_name	Ҫṩ JS õĺ
	 * param[in] function		
	 * param[in] replace		Ƿ滻ͬƵĺ壬Ĭϲ滻
	 * return replace Ϊ true £ true ʾע滻ɹfalse ǲԤ֪Ϊreplace Ϊ false · true ʾעɹ false ʾѾע
	 */
	bool RegisterCppFunc(const CefString& function_name, CppFunction function, CefRefPtr<CefBrowser> browser, bool  replace = false);

	/**
	 * עһ־õ C++ 
	 * param[in] function_name	Ҫעĺ
	 */
	void UnRegisterCppFunc(const CefString& function_name, CefRefPtr<CefBrowser> browser);

	/**
	 * ִһѾעõ C++ ܵ JS ִʱã
	 * param[in] function_name	Ҫִеĺ
	 * param[in] params			ЯĲ
	 * param[in] js_callback_id	ص JS Ļص ID
	 * param[in] browser		browser ʵ
	 * return  true ʾִгɹ false ʾִʧܣܲ
	 */
	bool ExecuteCppFunc(const CefString& function_name, const CefString& params, int js_callback_id, CefRefPtr<CefBrowser> browser);

private:
	uint32						js_callback_id_ = 0;			// JS ˻ص
	uint32						cpp_callback_id_ = 0;			// C++ ˻ص

	RenderCallbackMap			render_callback_;				// JS ˻صĶӦб
	BrowserCallbackMap			browser_callback_;				// C++ ˻صĶӦб

	RenderRegisteredFunction	render_registered_function_;	//  JS Ѿעõĳ־úб
	BrowserRegisteredFunction	browser_registered_function_;	//  C++ Ѿעõĳ־úб
};

}